/*
Take Turns
Author: Dan (dan@dantom.com)
Version: 2 (2001-Apr-04)
Version: 1 (1999-Jun-09)

The TakeTurns data type can be used to run turn based games.  Just add a few
players and define actions to take place for the various events.  Consult
TakeTurns.html for detailed information.

*/

TakeTurns
	var
		players[0]
		mob/cur_player
		round      //1-based game round counter
		turn_count //1-based turn counter
		start_time //time when current turn began
		time_limit //maximum length of turn (in 1/10 seconds)
		halted

	proc
		AddPlayer(mob/M)
		DelPlayer(mob/M)
		NextTurn()
		Halt()
		Resume()
		Restart()

		//to be defined by external code
		BeginRound()
		BeginTurn()  
		EndTurn()
		ExpireTurn()     //called when time has run out
		WarnExpireTurn() //called when time is 3/4 out

TakeTurns/Halt()
	src.halted = 1

TakeTurns/Resume()
	src.halted = 0
	if(!src.cur_player && src.players.len) src.NextTurn()

TakeTurns/Restart()
	src.cur_player = null
	src.Resume()

TakeTurns/AddPlayer(mob/M)
	src.players.Add(M)
	if(!src.cur_player) src.NextTurn()

TakeTurns/DelPlayer(mob/M)
	if(src.cur_player == M) src.NextTurn(0) // prevent repeat for single-player games
	src.players.Remove(M)
	if(src.cur_player == M) src.cur_player = null

TakeTurns/NextTurn(repeat=1)
	if(src.halted) return

	usr = src.cur_player
	if(usr) src.EndTurn()

	src.turn_count++
	var/cur_pos = src.players.Find(src.cur_player)
	var/next_pos
	if(cur_pos==0 || cur_pos==src.players.len)
		next_pos = 1
		src.round++
		src.BeginRound()
		if(!src) return
		if(src.halted)
			src.cur_player = null
			return
	else 
		next_pos = cur_pos+1

	if(next_pos<=src.players.len && (repeat || cur_pos!=next_pos))
		src.start_time = world.time
		src.cur_player = src.players[next_pos]
		usr = src.cur_player
		src.BeginTurn()

		//enforce the time limit
		if(src.time_limit)
			var/this_turn = src.turn_count
			spawn(src.time_limit * 0.75)
				if(this_turn != src.turn_count) return
				src.WarnExpireTurn()

				spawn(src.time_limit * 0.25)
					if(this_turn != src.turn_count) return
					src.ExpireTurn()
					src.NextTurn()

TakeTurns/BeginTurn()
	src.players << "It is [usr]'s turn."

TakeTurns/WarnExpireTurn()
	usr << "Your turn is about to expire."

TakeTurns/ExpireTurn()
	src.players << "[usr] runs out of time."
